/**
 * Program Name: createScript.java
 * Comments:
 * This is a parent class for all of the createJob scripts for the different
 * platforms.  It receives the ContextManager object from the calling JSP page
 * (or whatever).  This object is protected, and can be accesses by the 
 * children.  
 * 
 * We want to write the script to the user's "Descriptors" directory, which 
 * is located under the WEB-INF directory.  It is thus not accessible by
 * the web server and thus by the JSP pages.  So we have to use a bean.
 *
 * The String scriptString is used to store the script as it is constructed.
 * Part of this occurs here (the createCommands() method) and part must 
 * occur in the appropriate child (the createHeader() method).  So this 
 * string is protected and inherited by the child classes.
 *
 * I am assuming the following naming convention for the ContextData:
 * Directory: the user's decriptors directory (xml).
 * IOStyle: type of I/O.  Can by C-style, standard I/O, or internal(xml).
 * WorkDir: name of the workspace directory in which the program will
 *          execute (xml).
 * Name: name of problem (user).
 * NumInParams: number of input parameters (xml).
 * NumOutFiles: number of output files (xml).
 * inputParam: values of the input parameters, include input filenames.
 *             Possibly multivalues. (user or xml).
 * outputFiles: names of the output files.  Possible multivalued (user or xml).
 * execpath: full path of the executable (xml).
 * 
 * The data source can either be from ApplDesc.xml (xml) or from the 
 * user (user).
 */

package extensions.generic;

import WebFlow.*;
import WebFlow.ContextManager.*;
import java.io.*;

public abstract class createScript {
    protected ContextManager cm;
    protected String descDir;
    protected PrintWriter pw;
    protected scriptString="";

    public createScript(ContextManager cm) {
	this.cm=cm;
	descDir=cm.getCurrentProblemProperty("Directory");
	try {
	    pw=new PrintWriter(new FileWriter(descDir+"/job.script"));
	}
	catch(Exception ex) {
	    System.err.println("Could not open file for printing");
	    ex.printStackTrace();
	}
    }

    /**
     * This method should be used to instantiate the child classes.
     */
    public createScript scriptFactory() {
	return this;
    }
    
    /** 
     * The child should construct the script as a single string and
     * use the following method to print it out.
     */
    public void writeScript(String scriptString) {
	pw.print(scriptString);
	pw.flush();
	pw.close();
    }

    /**
     * The following method is used to construct the header information
     * with the queue specific directives.  For example, a PBS script
     * begins with
     *   #PBS -q default
     *   #PBS -l walltime=10:00:00
     * and so on.  Must be overriden by each child.
     */
    public abstract void createHeader();

    /**
     * The following method should be used to create the command portion
     * of the job script.  This part is code specific, not queue specific,
     * so we can do this here.
     * 
     * Note this requires that the ContextData has the following properties
     * set: IOStylem, WorkDir, Name.  This is FRAGILE since changes in 
     * the JSP pages that collect info and store them in the ContextData can
     * potentially break this method.  Currently we have to really on 
     * a name convention.  We have no way of enforcing this convention.  
     * Probably should move to XML/DTD for this reason.
     */
    public createCommands() {
	String IOStyle=cm.getCurrentProblemProperty("IOStyle");
	String workDir=cm.getCurrentProblemProperty("WorkDir");
	String jobName=cm.getCurrentProblemProperty("Name");
	int inArg=
	    Integer.parseInt(cm.getCurrentProblemProperty("NumInParams"));
	int outArg=
	    Integer.parseInt(cm.getCurrentProblemProperty("NumOutFiles"));
	String inSeparator,outSeparator;
	String[] inputParam=parseMultiVal("inputParam",inArg);
	String[] outputFile=parseMultiVal("outputFiles",outArg);

	//Set up the working directory
	workDir+="/"+jobName;
	scriptString+="if(! -d "+workDir+") then\n";
	scriptString+="  mkdir -p "+workDir+"\n";
	scriptString+="endif\n";
	scriptString+="cd "+workDir+"\n";

	if(IOStyle.equals("CArgument")){
	    inSeparator=" ";
	    outSeparator=" ";
	}
	else if(IOStyle.equals("StandardIO")){
	    inSeparator=" < ";
	    outSeparator=" > ";
	}
	else if(IOStyle.equals("Internal")){
	}
	else {
	    System.err.println("Invalid IOStyle detected");
	}

	scriptString+=cm.getCurrentProblemProperty("execpath");
	for(int i=0;i<inArg;i++) {
	    scriptString=scriptString+inSeparator+inputParam[i];
	}
	for(int j=0;j<outArg;j++) {
	    scriptString=scriptString+inSeparator+outputFiles[j];
	}
	scriptString+="\n";
	scriptString+="cd\n";
	scriptString+="rm -rf "+workDir;
    }   
    
    /**
     * ContextData is basically a hashtable, but often we need to store
     * multiple values for a single name.  For example, if we have multiple
     * input files, we would like to store them under a single name and 
     * restore them in an array.  
     * 
     * We will adopt the convention (following the way child contexts are
     * handled) by separating multiple values with a "+" sign.  For example,
     * we may have
     *             inputParam infile1+infile2+infile3
     */
    private String[] getMultiVal(String cdParamName,String dim){
	String toEat=cm.getCurrentProblemProperty(cdParamName);
	String[] multiVal=new String[dim];
	StringTokenizer st=new StringTokenizer(toEat,"+");
	for(int i=0;i<dim;i++) {
	    multiVal[i]=st.nextToken();
	}
	return multiVal;
    }

}
